// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/events.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/store.dart';
import 'package:matrix_sdk/src/util/json.dart';

class Room extends Identifiable<RoomId> implements JsonWritable {
  @override
  final RoomId id;

  final Events events;

  Room(this.id, {Store store, String name, Uri avatarUrl})
      : _name = name,
        _avatarUrl = avatarUrl,
        events = Events(id, store: store);

  String _name;
  String get name => _name;

  Uri _avatarUrl;
  Uri get avatarUrl => _avatarUrl;

  factory Room.fromJson(RoomId id, Map<String, dynamic> json) {
    final String name = json['name'];
    final Uri avatarUrl = Uri.parse(json['avatar_url']);
    return Room(id, name: name, avatarUrl: avatarUrl);
  }

  // TODO: Generate
  dynamic toJson() => { 'name': name, 'avatar_url': avatarUrl };

  void addEvents(List<dynamic> rawEvents) {
    for (var raw in rawEvents) {
      var event = Event.fromJson(this.id, raw);

      if (event != null) {
        events[event.id] = event;
      }

      if (event is NameChangeEvent) {
        _name = event.name;
      }

      if (event is AvatarChangeEvent) {
        _avatarUrl = event.url;
      }
    }
  }
}

class JoinedRoom extends Room {
  JoinedRoom(RoomId id, {Store store, String name})
      : super(id, name: name, store: store);

  factory JoinedRoom.fromJson(
      RoomId id, Store store, Map<String, dynamic> json) {
    final String name = json['name'];
    return JoinedRoom(id, name: name, store: store);
  }
}
