// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/store.dart';

/// Rooms map that will lazily load from the given
/// [store].
class Rooms {
  final Map<RoomId, Room> _memory = Map();

  final Store store;

  Rooms({this.store});

  Future<Room> operator [](RoomId key) async {
    // If the room is in memory, return that one
    var fromMemory = _memory[key];
    if (fromMemory != null) {
      return fromMemory;
    }

    // Look in the store
    return await store?.getJoinedRoom(key);
  }

  void operator []=(RoomId key, Room value) async {
    _memory[key] = value;

    await store?.setJoinedRoom(key, value);
  }

  Stream<Room> all() async* {
    for (var room in _memory.values) {
      yield room;
    }

    // Load all rooms to memory
    await for (var room in store?.getJoinedRooms()) {
      bool shouldYield = false;

      // If the room was not already in memory, yield it
      _memory.putIfAbsent(room.id, () {
        shouldYield = true;
        return room;
      });

      if (shouldYield) {
        yield room;
      }
    }
  }
}
